@@ -12,6 +12,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import copy
 import functools
 import os
 
@@ -29,6 +30,7 @@ from taskflow.patterns import linear_flow as lf
 from taskflow import retry
 from taskflow import task
 
+from glance.api import common as api_common
 import glance.async_.flows._internal_plugins as internal_plugins
 import glance.async_.flows.plugins as import_plugins
 from glance.common import exception
@@ -192,6 +194,18 @@ class _ImportActions(object):
     def image_id(self):
         return self._image.image_id
 
+    @property
+    def image_size(self):
+        return self._image.size
+
+    @property
+    def image_locations(self):
+        # Return a copy of this complex structure to make sure we do
+        # not allow the plugin to mutate this underneath us for our
+        # later save.  If this needs to be a thing in the future, we
+        # should have moderated access like all the other things here.
+        return copy.deepcopy(self._image.locations)
+
     @property
     def image_status(self):
         return self._image.status
@@ -284,12 +298,40 @@ class _ImportActions(object):
                                            set_active=set_active,
                                            callback=callback)
 
-    def set_image_status(self, status):
-        """Set the image status.
+    def set_image_attribute(self, **attrs):
+        """Set an image attribute.
 
-        :param status: The new status of the image
+        This allows setting various image attributes which will be saved
+        upon exiting the ImportActionWrapper context.
+
+        :param attrs: kwarg list of attributes to set on the image
+        :raises: AttributeError if an attribute outside the set of allowed
+                 ones is present in attrs.
         """
-        self._image.status = status
+        allowed = ['status', 'disk_format', 'container_format',
+                   'virtual_size']
+        for attr, value in attrs.items():
+            if attr not in allowed:
+                raise AttributeError('Setting %s is not allowed' % attr)
+            setattr(self._image, attr, value)
+
+    def set_image_extra_properties(self, properties):
+        """Merge values into image extra_properties.
+
+        This allows a plugin to set additional properties on the image,
+        as long as those are outside the reserved namespace. Any keys
+        in the internal namespace will be dropped (and logged).
+
+        :param properties: A dict of properties to be merged in
+        """
+        for key, value in properties.items():
+            if key.startswith(api_common.GLANCE_RESERVED_NS):
+                LOG.warning(('Dropping %(key)s=%(val)s during metadata '
+                             'injection for %(image)s'),
+                            {'key': key, 'val': value,
+                             'image': self.image_id})
+            else:
+                self._image.extra_properties[key] = value
 
     def remove_location_for_store(self, backend):
         """Remove a location from an image given a backend store.
@@ -324,6 +366,15 @@ class _ImportActions(object):
                         self._image.size = None
                 break
 
+    def pop_extra_property(self, name):
+        """Delete the named extra_properties value, if present.
+
+        If the image.extra_properties dict contains the named key,
+        delete it.
+        :param name: The key to delete.
+        """
+        self._image.extra_properties.pop(name, None)
+
 
 class _DeleteFromFS(task.Task):
 
@@ -627,7 +678,7 @@ class _VerifyImageState(task.Task):
         """Set back to queued if this wasn't copy-image job."""
         with self.action_wrapper as action:
             if self.import_method != 'copy-image':
-                action.set_image_status('queued')
+                action.set_image_attribute(status='queued')
 
 
 class _CompleteTask(task.Task):
@@ -717,6 +768,7 @@ def get_flow(**kwargs):
     # otherwise with the regular repo.
     action_wrapper = ImportActionWrapper(admin_repo or image_repo, image_id,
                                          task_id)
+    kwargs['action_wrapper'] = action_wrapper
 
     if not uri and import_method in ['glance-direct', 'copy-image']:
         if CONF.enabled_backends:
@@ -730,8 +782,7 @@ def get_flow(**kwargs):
     flow.add(_ImageLock(task_id, task_type, action_wrapper))
 
     if import_method in ['web-download', 'copy-image']:
-        internal_plugin = internal_plugins.get_import_plugin(
-            **dict(kwargs, action_wrapper=action_wrapper))
+        internal_plugin = internal_plugins.get_import_plugin(**kwargs)
         flow.add(internal_plugin)
         if CONF.enabled_backends:
             separator, staging_dir = store_utils.get_dir_separator()
@@ -786,8 +837,9 @@ def get_flow(**kwargs):
 
     with action_wrapper as action:
         if import_method != 'copy-image':
-            action.set_image_status('importing')
+            action.set_image_attribute(status='importing')
         action.add_importing_stores(stores)
         action.remove_failed_stores(stores)
+        action.pop_extra_property('os_glance_stage_host')
 
     return flow
