@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import atexit
 import os
 import threading
 
@@ -19,9 +20,10 @@ from oslo_log import log as logging
 import osprofiler.initializer
 
 from glance.api import common
+import glance.async_
 from glance.common import config
+from glance.common import exception
 from glance.common import store_utils
-import glance.db
 from glance import housekeeping
 from glance.i18n import _
 from glance import notifier
@@ -70,6 +72,16 @@ def _setup_os_profiler():
                                               host=CONF.bind_host)
 
 
+def _validate_policy_enforcement_configuration():
+    if CONF.enforce_secure_rbac != CONF.oslo_policy.enforce_new_defaults:
+        fail_message = (
+            "[DEFAULT] enforce_secure_rbac does not match "
+            "[oslo_policy] enforce_new_defaults. Please set both to "
+            "True to enable secure RBAC personas. Otherwise, make sure "
+            "both are False.")
+        raise exception.ServerError(fail_message)
+
+
 def drain_threadpools():
     # NOTE(danms): If there are any other named pools that we need to
     # drain before exit, they should be in this list.
@@ -97,6 +109,18 @@ def init_app():
     CONF([], project='glance', default_config_files=config_files)
     logging.setup(CONF, "glance")
 
+    # NOTE(danms): We are running inside uwsgi or mod_wsgi, so no eventlet;
+    # use native threading instead.
+    glance.async_.set_threadpool_model('native')
+    atexit.register(drain_threadpools)
+
+    # NOTE(danms): Change the default threadpool size since we
+    # are dealing with native threads and not greenthreads.
+    # Right now, the only pool of default size is tasks_pool,
+    # so if others are created this will need to change to be
+    # more specific.
+    common.DEFAULT_POOL_SIZE = CONF.wsgi.task_pool_threads
+
     if CONF.enabled_backends:
         if store_utils.check_reserved_stores(CONF.enabled_backends):
             msg = _("'os_glance_' prefix should not be used in "
@@ -114,4 +138,5 @@ def init_app():
     run_staging_cleanup()
 
     _setup_os_profiler()
+    _validate_policy_enforcement_configuration()
     return config.load_paste_app('glance-api')
