@@ -151,19 +151,34 @@ def image_create(context, values, v1_mode=False):
 
 
 def image_update(context, image_id, values, purge_props=False,
-                 from_state=None, v1_mode=False):
+                 from_state=None, v1_mode=False, atomic_props=None):
     """
     Set the given properties on an image and update it.
 
     :raises: ImageNotFound if image does not exist.
     """
     image = _image_update(context, values, image_id, purge_props,
-                          from_state=from_state)
+                          from_state=from_state, atomic_props=atomic_props)
     if v1_mode:
         image = db_utils.mutate_image_dict_to_v1(image)
     return image
 
 
+def image_restore(context, image_id):
+    """Restore the pending-delete image to active."""
+    session = get_session()
+    with session.begin():
+        image_ref = _image_get(context, image_id, session=session)
+        if image_ref.status != 'pending_delete':
+            msg = (_('cannot restore the image from %s to active (wanted '
+                     'from_state=pending_delete)') % image_ref.status)
+            raise exception.Conflict(msg)
+
+        query = session.query(models.Image).filter_by(id=image_id)
+        values = {'status': 'active', 'deleted': 0}
+        query.update(values, synchronize_session='fetch')
+
+
 @retry(retry_on_exception=_retry_on_deadlock, wait_fixed=500,
        stop_max_attempt_number=50)
 def image_destroy(context, image_id):
@@ -445,10 +460,18 @@ def _make_conditions_from_filters(filters, is_public=None):
         else:
             image_conditions.append(models.Image.visibility != 'public')
 
+    if 'os_hidden' in filters:
+        os_hidden = filters.pop('os_hidden')
+        image_conditions.append(models.Image.os_hidden == os_hidden)
+
     if 'checksum' in filters:
         checksum = filters.pop('checksum')
         image_conditions.append(models.Image.checksum == checksum)
 
+    if 'os_hash_value' in filters:
+        os_hash_value = filters.pop('os_hash_value')
+        image_conditions.append(models.Image.os_hash_value == os_hash_value)
+
     for (k, v) in filters.pop('properties', {}).items():
         prop_filters = _make_image_property_condition(key=k, value=v)
         prop_conditions.append(prop_filters)
@@ -635,8 +658,9 @@ def image_get_all(context, filters=None, marker=None, limit=None,
                                  visibility)
     if visibility is not None:
         # with a visibility, we always and only include images with that
-        # visibility
-        query = query.filter(models.Image.visibility == visibility)
+        # visibility except when using the 'all' visibility
+        if visibility != 'all':
+            query = query.filter(models.Image.visibility == visibility)
     elif context.owner is None:
         # without either a visibility or an owner, we never include
         # 'community' images
@@ -754,17 +778,107 @@ def _update_values(image_ref, values):
             setattr(image_ref, k, values[k])
 
 
+def image_set_property_atomic(image_id, name, value):
+    """
+    Atomically set an image property to a value.
+
+    This will only succeed if the property does not currently exist
+    and it was created successfully. This can be used by multiple
+    competing threads to ensure that only one of those threads
+    succeeded in creating the property.
+
+    Note that ImageProperty objects are marked as deleted=$id and so we must
+    first try to atomically update-and-undelete such a property, if it
+    exists. If that does not work, we should try to create the property. The
+    latter should fail with DBDuplicateEntry because of the UniqueConstraint
+    across ImageProperty(image_id, name).
+
+    :param image_id: The ID of the image on which to create the property
+    :param name: The property name
+    :param value: The value to set for the property
+    :raises Duplicate: If the property already exists
+    """
+    session = get_session()
+    with session.begin():
+        connection = session.connection()
+        table = models.ImageProperty.__table__
+
+        # This should be:
+        #   UPDATE image_properties SET value=$value, deleted=0
+        #     WHERE name=$name AND deleted!=0
+        result = connection.execute(table.update().where(
+            sa_sql.and_(table.c.name == name,
+                        table.c.image_id == image_id,
+                        table.c.deleted != 0)).values(
+                            value=value, deleted=0))
+        if result.rowcount == 1:
+            # Found and updated a deleted property, so we win
+            return
+
+        # There might have been no deleted property, or the property
+        # exists and is undeleted, so try to create it and use that
+        # to determine if we've lost the race or not.
+
+        try:
+            connection.execute(table.insert(),
+                               dict(deleted=False,
+                                    created_at=timeutils.utcnow(),
+                                    image_id=image_id,
+                                    name=name,
+                                    value=value))
+        except db_exception.DBDuplicateEntry:
+            # Lost the race to create the new property
+            raise exception.Duplicate()
+
+        # If we got here, we created a new row, UniqueConstraint would have
+        # caused us to fail if we lost the race
+
+
+def image_delete_property_atomic(image_id, name, value):
+    """
+    Atomically delete an image property.
+
+    This will only succeed if the referenced image has a property set
+    to exactly the value provided.
+
+    :param image_id: The ID of the image on which to delete the property
+    :param name: The property name
+    :param value: The value the property is expected to be set to
+    :raises NotFound: If the property does not exist
+    """
+    session = get_session()
+    with session.begin():
+        connection = session.connection()
+        table = models.ImageProperty.__table__
+
+        result = connection.execute(table.delete().where(
+            sa_sql.and_(table.c.name == name,
+                        table.c.value == value,
+                        table.c.image_id == image_id,
+                        table.c.deleted == 0)))
+        if result.rowcount == 1:
+            return
+
+        raise exception.NotFound()
+
+
 @retry(retry_on_exception=_retry_on_deadlock, wait_fixed=500,
        stop_max_attempt_number=50)
 @utils.no_4byte_params
 def _image_update(context, values, image_id, purge_props=False,
-                  from_state=None):
+                  from_state=None, atomic_props=None):
     """
     Used internally by image_create and image_update
 
     :param context: Request context
     :param values: A dict of attributes to set
     :param image_id: If None, create the image, otherwise, find and update it
+    :param from_state: If not None, reequire the image be in this state to do
+                       the update
+    :param purge_props: If True, delete properties found in the database but
+                        not present in values
+    :param atomic_props: If non-None, refuse to create or update properties
+                         in this list
     """
 
     # NOTE(jbresnah) values is altered in this so a copy is needed
@@ -855,7 +969,7 @@ def _image_update(context, values, image_id, purge_props=False,
                                           % values['id'])
 
         _set_properties_for_image(context, image_ref, properties, purge_props,
-                                  session)
+                                  atomic_props, session)
 
         if location_data:
             _image_locations_set(context, image_ref.id, location_data,
@@ -970,15 +1084,24 @@ def _image_locations_delete_all(context, image_id,
 
 @utils.no_4byte_params
 def _set_properties_for_image(context, image_ref, properties,
-                              purge_props=False, session=None):
+                              purge_props=False, atomic_props=None,
+                              session=None):
     """
     Create or update a set of image_properties for a given image
 
     :param context: Request context
     :param image_ref: An Image object
     :param properties: A dict of properties to set
+    :param purge_props: If True, delete properties in the database
+                        that are not in properties
+    :param atomic_props: If non-None, skip update/create/delete of properties
+                         named in this list
     :param session: A SQLAlchemy session to use (if present)
     """
+
+    if atomic_props is None:
+        atomic_props = []
+
     orig_properties = {}
     for prop_ref in image_ref.properties:
         orig_properties[prop_ref.name] = prop_ref
@@ -987,7 +1110,11 @@ def _set_properties_for_image(context, image_ref, properties,
         prop_values = {'image_id': image_ref.id,
                        'name': name,
                        'value': value}
-        if name in orig_properties:
+        if name in atomic_props:
+            # NOTE(danms): Never update or create properties in the list
+            # of atomics
+            continue
+        elif name in orig_properties:
             prop_ref = orig_properties[name]
             _image_property_update(context, prop_ref, prop_values,
                                    session=session)
@@ -996,7 +1123,9 @@ def _set_properties_for_image(context, image_ref, properties,
 
     if purge_props:
         for key in orig_properties.keys():
-            if key not in properties:
+            if key in atomic_props:
+                continue
+            elif key not in properties:
                 prop_ref = orig_properties[key]
                 image_property_delete(context, prop_ref.name,
                                       image_ref.id, session=session)
@@ -1299,6 +1428,35 @@ def purge_deleted_rows(context, age_in_days, max_rows, session=None):
             continue
         if hasattr(model_class, 'deleted'):
             tables.append(model_class.__tablename__)
+
+    # First force purging of records that are not soft deleted but
+    # are referencing soft deleted tasks/images records (e.g. task_info
+    # records). Then purge all soft deleted records in glance tables in the
+    # right order to avoid FK constraint violation.
+    t = Table("tasks", metadata, autoload=True)
+    ti = Table("task_info", metadata, autoload=True)
+    joined_rec = ti.join(t, t.c.id == ti.c.task_id)
+    deleted_task_info = sql.select([ti.c.task_id],
+                                   t.c.deleted_at < deleted_age).\
+        select_from(joined_rec).order_by(t.c.deleted_at).limit(max_rows)
+    delete_statement = DeleteFromSelect(ti, deleted_task_info,
+                                        ti.c.task_id)
+    LOG.info(_LI('Purging deleted rows older than %(age_in_days)d day(s) '
+                 'from table %(tbl)s'),
+             {'age_in_days': age_in_days, 'tbl': ti})
+    try:
+        with session.begin():
+            result = session.execute(delete_statement)
+    except (db_exception.DBError, db_exception.DBReferenceError) as ex:
+        LOG.exception(_LE('DBError detected when force purging '
+                          'table=%(table)s: %(error)s'),
+                      {'table': ti, 'error': six.text_type(ex)})
+        raise
+
+    rows = result.rowcount
+    LOG.info(_LI('Deleted %(rows)d row(s) from table %(tbl)s'),
+             {'rows': rows, 'tbl': ti})
+
     # get rid of FK constraints
     for tbl in ('images', 'tasks'):
         try:
@@ -1307,6 +1465,11 @@ def purge_deleted_rows(context, age_in_days, max_rows, session=None):
             LOG.warning(_LW('Expected table %(tbl)s was not found in DB.'),
                         {'tbl': tbl})
         else:
+            # NOTE(abhishekk): To mitigate OSSN-0075 images records should be
+            # purged with new ``purge-images-table`` command.
+            if tbl == 'images':
+                continue
+
             tables.append(tbl)
 
     for tbl in tables:
@@ -1339,6 +1502,50 @@ def purge_deleted_rows(context, age_in_days, max_rows, session=None):
                  {'rows': rows, 'tbl': tbl})
 
 
+def purge_deleted_rows_from_images(context, age_in_days, max_rows,
+                                   session=None):
+    """Purges soft deleted rows
+
+    Deletes rows of table images table according to given age for
+    relevant models.
+    """
+    # check max_rows for its maximum limit
+    _validate_db_int(max_rows=max_rows)
+
+    session = session or get_session()
+    metadata = MetaData(get_engine())
+    deleted_age = timeutils.utcnow() - datetime.timedelta(days=age_in_days)
+
+    tbl = 'images'
+    tab = Table(tbl, metadata, autoload=True)
+    LOG.info(
+        _LI('Purging deleted rows older than %(age_in_days)d day(s) '
+            'from table %(tbl)s'),
+        {'age_in_days': age_in_days, 'tbl': tbl})
+
+    column = tab.c.id
+    deleted_at_column = tab.c.deleted_at
+
+    query_delete = sql.select(
+        [column], deleted_at_column < deleted_age).order_by(
+        deleted_at_column).limit(max_rows)
+
+    delete_statement = DeleteFromSelect(tab, query_delete, column)
+
+    try:
+        with session.begin():
+            result = session.execute(delete_statement)
+    except db_exception.DBReferenceError as ex:
+        with excutils.save_and_reraise_exception():
+            LOG.error(_LE('DBError detected when purging from '
+                      "%(tablename)s: %(error)s"),
+                      {'tablename': tbl, 'error': six.text_type(ex)})
+
+    rows = result.rowcount
+    LOG.info(_LI('Deleted %(rows)d row(s) from table %(tbl)s'),
+             {'rows': rows, 'tbl': tbl})
+
+
 def user_get_storage_usage(context, owner_id, image_id=None, session=None):
     _check_image_id(image_id)
     session = session or get_session()
@@ -1454,6 +1661,43 @@ def task_get(context, task_id, session=None, force_show_deleted=False):
     return _task_format(task_ref, task_ref.info)
 
 
+def tasks_get_by_image(context, image_id, session=None):
+    """Fetch all tasks associated with image_id"""
+    tasks = []
+    session = session or get_session()
+    _task_soft_delete(context, session=session)
+
+    query = session.query(models.Task).options(
+        sa_orm.joinedload(models.Task.info)
+    ).filter_by(image_id=image_id)
+
+    expires_at = models.Task.expires_at
+    query = query.filter(expires_at >= timeutils.utcnow())
+    updated_at = models.Task.updated_at
+    query.filter(
+        updated_at <= (timeutils.utcnow() +
+                       datetime.timedelta(hours=CONF.task.task_time_to_live)))
+
+    if not context.can_see_deleted:
+        query = query.filter_by(deleted=False)
+
+    try:
+        task_refs = query.all()
+    except sa_orm.exc.NoResultFound:
+        LOG.debug("No task found for image with ID %s", image_id)
+        return tasks
+
+    for task_ref in task_refs:
+        # Make sure the task is visible
+        if not _is_task_visible(context, task_ref):
+            msg = "Task %s is not visible, excluding" % task_ref.id
+            LOG.debug(msg)
+            continue
+        tasks.append(_task_format(task_ref, task_ref.info))
+
+    return tasks
+
+
 def task_delete(context, task_id, session=None):
     """Delete a task"""
     session = session or get_session()
@@ -1600,7 +1844,10 @@ def _task_format(task_ref, task_info_ref=None):
         'created_at': task_ref['created_at'],
         'updated_at': task_ref['updated_at'],
         'deleted_at': task_ref['deleted_at'],
-        'deleted': task_ref['deleted']
+        'deleted': task_ref['deleted'],
+        'image_id': task_ref['image_id'],
+        'request_id': task_ref['request_id'],
+        'user_id': task_ref['user_id'],
     }
 
     if task_info_ref:
