@@ -14,13 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-# TODO(smcginnis) update this once six has support for collections.abc
-# (https://github.com/benjaminp/six/pull/241) or clean up once we drop py2.7.
-try:
-    from collections.abc import MutableMapping
-except ImportError:
-    from collections import MutableMapping
-
+from collections import abc
 import datetime
 import uuid
 
@@ -293,7 +287,7 @@ class Image(object):
         raise NotImplementedError()
 
 
-class ExtraProperties(MutableMapping, dict):
+class ExtraProperties(abc.MutableMapping, dict):
 
     def __getitem__(self, key):
         return dict.__getitem__(self, key)
@@ -362,6 +356,7 @@ class Task(object):
     _supported_task_status = ('pending', 'processing', 'success', 'failure')
 
     def __init__(self, task_id, task_type, status, owner,
+                 image_id, user_id, request_id,
                  expires_at, created_at, updated_at,
                  task_input, result, message):
 
@@ -385,6 +380,9 @@ class Task(object):
         self.task_input = task_input
         self.result = result
         self.message = message
+        self.image_id = image_id
+        self.request_id = request_id
+        self.user_id = user_id
 
     @property
     def status(self):
@@ -455,7 +453,8 @@ class Task(object):
 class TaskStub(object):
 
     def __init__(self, task_id, task_type, status, owner,
-                 expires_at, created_at, updated_at):
+                 expires_at, created_at, updated_at,
+                 image_id, user_id, request_id):
         self.task_id = task_id
         self._status = status
         self.type = task_type
@@ -463,6 +462,9 @@ class TaskStub(object):
         self.expires_at = expires_at
         self.created_at = created_at
         self.updated_at = updated_at
+        self.image_id = image_id
+        self.request_id = request_id
+        self.user_id = user_id
 
     @property
     def status(self):
@@ -471,8 +473,8 @@ class TaskStub(object):
 
 class TaskFactory(object):
 
-    def new_task(self, task_type, owner,
-                 task_input=None, **kwargs):
+    def new_task(self, task_type, owner, image_id, user_id,
+                 request_id, task_input=None, **kwargs):
         task_id = str(uuid.uuid4())
         status = 'pending'
         # Note(nikhil): expires_at would be set on the task, only when it
@@ -485,22 +487,26 @@ class TaskFactory(object):
             task_type,
             status,
             owner,
+            image_id,
+            user_id,
+            request_id,
             expires_at,
             created_at,
             updated_at,
             task_input,
             kwargs.get('result'),
-            kwargs.get('message')
+            kwargs.get('message'),
         )
 
 
 class TaskExecutorFactory(object):
     eventlet_deprecation_warned = False
 
-    def __init__(self, task_repo, image_repo, image_factory):
+    def __init__(self, task_repo, image_repo, image_factory, admin_repo=None):
         self.task_repo = task_repo
         self.image_repo = image_repo
         self.image_factory = image_factory
+        self.admin_repo = admin_repo
 
     def new_task_executor(self, context):
         try:
@@ -526,7 +532,8 @@ class TaskExecutorFactory(object):
             return executor(context,
                             self.task_repo,
                             self.image_repo,
-                            self.image_factory)
+                            self.image_factory,
+                            admin_repo=self.admin_repo)
         except ImportError:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE("Failed to load the %s executor provided "
