@@ -13,15 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import absolute_import
-
 from contextlib import contextmanager
 import datetime
-import hashlib
 import os
 import time
 
 import fixtures
+from oslo_utils import secretutils
 from oslo_utils import units
 import six
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
@@ -29,8 +27,6 @@ from six.moves import range
 
 from glance.common import exception
 from glance import image_cache
-# NOTE(bcwaldon): This is imported to load the registry config options
-import glance.registry  # noqa
 from glance.tests import utils as test_utils
 from glance.tests.utils import skip_if_disabled
 from glance.tests.utils import xattr_writes_supported
@@ -413,7 +409,7 @@ class ImageCacheTestCase(object):
         image = b"12345678990abcdefghijklmnop"
         image_id = 123
 
-        md5 = hashlib.md5()
+        md5 = secretutils.md5(usedforsecurity=False)
         md5.update(image)
         checksum = md5.hexdigest()
 
